def docs() -> str:
    return """
Insere um novo atendimento avulso no sistema SIGA para a área de Infraestrutura.

INSTRUÇÃO PARA O AGENTE IA:
ANTES de executar esta função de inserção:
1. MOSTRE ao usuário TODAS as informações que serão criadas/inseridas no sistema
2. APRESENTE os dados de forma clara e organizada (datas, descrição, tipo, origem, categoria, equipe, projeto, analista, plaqueta, etc.)
3. PEÇA confirmação explícita do usuário: "Confirma a criação? (sim/não)"
4. SÓ EXECUTE esta função se o usuário confirmar explicitamente
5. Se o usuário não confirmar, cancele a operação e informe que foi cancelada

Esta função cria um novo registro de atendimento avulso independente de qualquer Ordem de Serviço,
incluindo informações como datas, descrição, tipo, origem, categoria, equipe, projeto e plaqueta.
Realiza validação de todos os campos obrigatórios e conversão automática de datas.

ORIENTAÇÃO PARA SOLICITAÇÕES GENÉRICAS:
Quando o usuário solicitar "inserir/criar um atendimento" sem especificar o tipo:
1. Perguntar qual tipo: Atendimento OS, Atendimento Avulso Sistemas, ou Atendimento Avulso Infraestrutura
2. Se escolher "Avulso": Perguntar se é de Sistemas ou Infraestrutura
3. Depois de definir o tipo: Direcionar para a função específica correspondente

Esta função é específica para Atendimentos Avulso Infraestrutura. Para outros tipos, use:
- `inserir_atendimentos_os` (vinculados a OS)  
- `inserir_atendimento_avulso_sistemas` (área de Sistemas)

Args:
    data_inicio: Data e hora de início do atendimento avulso.
        Aceita formatos de data ou palavras-chave como "hoje", "agora", "ontem"
    data_fim: Data e hora de fim do atendimento avulso.
        Aceita formatos de data ou palavras-chave como "hoje", "agora", "ontem"
    matricula_solicitante: Matrícula do usuário que está solicitando o atendimento avulso
    descricao_atendimento: Descrição detalhada do atendimento avulso a ser realizado
    codigo_analista: Matrícula do analista/usuário responsável pelo atendimento avulso.
    Deve ser "CURRENT_USER" ou um número válido maior que zero. Não aceita valores vazios, "0" ou não-numéricos.
    tipo: Tipo do atendimento avulso, deve ser um dos valores válidos.
    origem: Canal/origem do atendimento avulso, deve ser um dos valores válidos.
    categoria: Categoria relacionado ao atendimento avulso, deve ser um dos valores válidos.
    equipe: Equipe responsável pelo atendimento avulso, deve ser uma das equipes válidas.
    projeto: Projeto relacionado ao atendimento avulso, deve ser um dos valores válidos.
    plaqueta: Plaqueta relacionado ao equipamento que está sendo atendido.

Notes:
    - DIFERENCIAÇÃO DE FUNÇÃO: Esta função cria novos atendimentos avulso infraestrutura. Para editar, use editar_atendimento_avulso_infraestrutura
    - VALIDAÇÃO OBRIGATÓRIA: Campo codigo_analista deve ser "CURRENT_USER" ou um número válido maior que zero (não aceita valores vazios, "0" ou não-numéricos)
    - Esta função é específica para ÁREA INFRAESTRUTURA (área=2)
    - A função realiza validação case-insensitive de todos os campos (tipo, origem, categoria, equipe, projeto)
    - As datas são automaticamente convertidas usando converter_data_siga com manter_horas=True
    - Utiliza as constantes: TIPO_TO_NUMBER_ATENDIMENTO_AVULSO_INFRAESTRUTURA, ORIGEM_TO_NUMBER, CATEGORIA_TO_NUMBER, EQUIPE_INFRAESTRUTURA_TO_NUMBER, PROJETO_TO_NUMBER para mapear valores para códigos numéricos
    - Todos os parâmetros enviados são incluídos como atributos no XML de resposta
    - A API key é obtida automaticamente da variável de ambiente AVA_API_KEY
    - Em caso de falha na requisição HTTP, retorna erro interno formatado em XML
    - O resultado da API (1 = sucesso, outros valores = erro) é interpretado automaticamente
    - Campos opcionais como nomeSolicitante, centroCusto, etc. são enviados vazios por padrão
    - Esta função cria atendimentos avulsos independentes, não vinculados a nenhuma OS
"""



def docs() -> str:
    return """
Insere um novo atendimento em uma Ordem de Serviço (OS) no sistema SIGA.

INSTRUÇÃO PARA O AGENTE IA:
ANTES de executar esta função de inserção:
1. MOSTRE ao usuário TODAS as informações que serão criadas/inseridas no sistema
2. APRESENTE os dados de forma clara e organizada (OS, datas, descrição, tipo, analista, etc.)
3. PEÇA confirmação explícita do usuário: "Confirma a criação? (sim/não)"
4. SÓ EXECUTE esta função se o usuário confirmar explicitamente
5. Se o usuário não confirmar, cancele a operação e informe que foi cancelada

Esta função cria um novo registro de atendimento associado a uma OS existente,
incluindo informações como datas, descrição, tipo, tempo gasto e flags de controle.
Realiza validação do tipo de atendimento e conversão automática de datas.

ORIENTAÇÃO PARA SOLICITAÇÕES GENÉRICAS:
Quando o usuário solicitar "inserir/criar um atendimento" sem especificar o tipo:
1. Perguntar qual tipo: Atendimento OS, Atendimento Avulso Sistemas, ou Atendimento Avulso Infraestrutura
2. Se escolher "Avulso": Perguntar se é de Sistemas ou Infraestrutura
3. Depois de definir o tipo: Direcionar para a função específica correspondente

Esta função é específica para Atendimentos OS. Para outros tipos, use:
- `inserir_atendimento_avulso_sistemas` (área de Sistemas)  
- `inserir_atendimento_avulso_infraestrutura` (área de Infraestrutura)

Args:
    codigo_os: Código da Ordem de Serviço à qual o atendimento será associado
    data_inicio: Data e hora de início do atendimento. Aceita formatos de data ou palavras-chave como "hoje", "agora", "ontem"
    codigo_analista: Matrícula do analista/usuário responsável pelo atendimento. Deve ser um número maior que zero. Não aceita valores vazios, zero ou negativos.
    descricao_atendimento: Descrição detalhada do atendimento a ser realizado
    tipo: Tipo do atendimento, deve ser um dos valores válidos.
    data_fim: Data e hora de fim do atendimento. Aceita formatos de data ou palavras-chave como "hoje", "agora", "ontem". Se None, será enviado como string vazia. Defaults to None.
    primeiro_atendimento: Flag indicando se é o primeiro atendimento da OS.
    apresenta_solucao: Flag indicando se o atendimento apresenta solução.

Notes:
    - DIFERENCIAÇÃO DE FUNÇÃO: Esta função cria novos atendimentos OS. Para editar, use editar_atendimentos_os
    - VALIDAÇÃO OBRIGATÓRIA: Campo codigo_analista deve ser maior que zero (não aceita valores vazios, zero ou negativos)
    - A função realiza validação case-insensitive do tipo de atendimento
    - As datas são automaticamente convertidas usando converter_data_siga com manter_horas=True
    - A função utiliza a constante TYPE_TO_NUMBER para mapear tipos para códigos numéricos
    - Todos os parâmetros enviados são incluídos como atributos no XML de resposta
    - A API key é obtida automaticamente da variável de ambiente AVA_API_KEY
    - Em caso de falha na requisição HTTP, retorna erro interno formatado em XML
    - O resultado da API (1 = sucesso, outros valores = erro) é interpretado automaticamente
    - Esta função cria um novo atendimento, diferente de editar_atendimentos_os que modifica existente

"""



def docs() -> str:
    return """
Extrai matrículas de usuários do XML retornado pela função listar_usuarios_equipe_por_gerente.

Esta função utilitária assíncrona parseia o XML gerado por listar_usuarios_equipe_por_gerente e extrai
todas as matrículas encontradas, retornando uma lista limpa e pronta para uso em outras
funções do sistema. É especialmente útil para integração com a função listar_horas_trabalhadas,
permitindo buscar horas trabalhadas de toda uma equipe de forma otimizada.

A função trata automaticamente casos de XML malformado, elementos ausentes ou vazios,
garantindo sempre o retorno de uma lista válida (mesmo que vazia) sem quebrar o fluxo
de execução do código.

Args:
    xml_string: 
        String contendo o XML retornado pela função listar_usuarios_equipe_por_gerente.
        Deve estar no formato esperado com elementos <usuario_equipe_gerente> contendo
        subelementos <usuario> com as matrículas. Aceita qualquer XML válido,
        incluindo casos de erro, aviso ou resultado vazio.

Notes:
    - FUNÇÃO UTILITÁRIA ASSÍNCRONA: Projetada especificamente para integração entre funções de equipe e horas
    - ROBUSTEZ: Nunca falha - sempre retorna lista válida mesmo com XML inválido
    - PERFORMANCE: Parsing rápido usando xml.etree.ElementTree nativo do Python
    - INTEGRAÇÃO OTIMIZADA: Funciona perfeitamente com listar_horas_trabalhadas otimizada para listas
    - CASOS DE ERRO: XML de erro/aviso retorna lista vazia, permitindo tratamento adequado
    - LIMPEZA AUTOMÁTICA: Remove espaços em branco das matrículas automaticamente
    - XPATH USADO: './/usuario_equipe_gerente' para buscar todos os elementos independente da profundidade
    - VALIDAÇÃO: Verifica se elemento <usuario> existe e contém texto antes de adicionar à lista
    - FLUXO RECOMENDADO: 
      1. Chamar listar_usuarios_equipe_por_gerente
      2. Chamar await extrair_matriculas_do_xml com o resultado
      3. Verificar se lista não está vazia
      4. Chamar listar_horas_trabalhadas com a lista de matrículas
    - REUTILIZAÇÃO: Pode ser usada com qualquer XML que siga o padrão de usuarios_equipe_gerente
    - THREAD-SAFE: Função pura sem estado, segura para uso concorrente
    - MEMORIA: Eficiente - processa XML em streaming sem carregar tudo na memória
"""



import asyncio
from os import getenv
import sys
import os

# from siga_mcp.tools import inserir_os_infraestrutura
# from siga_mcp.dynamic_constants import obter_usuarios_responsavel
from siga_mcp.tools.genericas import (
    buscar_pendencias_multiplas_matriculas,
    buscar_pendencias_lancamentos_atendimentos,
)
from siga_mcp.tools.os import (
    buscar_informacoes_os,
    buscar_todas_os_usuario,
    inserir_os_sistemas,
)
from siga_mcp.dynamic_constants import buscar_constante_siga


from dotenv import load_dotenv

# 🔑 CARREGAR .ENV AUTOMATICAMENTE
load_dotenv()


def main():
    resultado = buscar_constante_siga(
        endpoint="https://ava3.uniube.br/ava/api/segProjetos/buscarProjetosOsSigaIA/",
        campo_chave="DESCRICAO_PROJETO",
        campo_valor="CODIGO_PROJETO",
        nome_constante="Projeto OS Sistemas",
    )
    return resultado


if __name__ == "__main__":
    resultado = main()
    print(resultado)

# separando os resultados
print("\n" + "=" * 70)


def main():
    resultado = buscar_constante_siga(
        endpoint="https://ava3.uniube.br/ava/api/seglinguagens/buscarLinguagensOsSigaIA/",
        campo_chave="DESCRICAO_LINGUAGEM",
        campo_valor="CODIGO_LINGUAGEM",
        nome_constante="Linguagens OS Sistemas",
    )
    return resultado


if __name__ == "__main__":
    resultado = main()
    print(resultado)

print("\n" + "=" * 70)


def main():
    resultado = buscar_constante_siga(
        endpoint="https://ava3.uniube.br/ava/api/segSistemas/buscarSistemasOsSigaIA/",
        campo_chave="DESCRICAO_SISTEMA",
        campo_valor="CODIGO_SISTEMA",
        nome_constante="Sistemas OS Sistemas",
    )
    return resultado


if __name__ == "__main__":
    resultado = main()
    print(resultado)

""" async def main() -> str:
    return await buscar_todas_os_usuario(
        **{"filtrar_por": "Todas OS em Aberto", "matricula": "24290"}
    )


if __name__ == "__main__":
    resultado = asyncio.run(main())
    print(resultado) """

"""def main() -> str:
return listar_usuarios_responsaveis_os_siga(
    **{
        "area": "1",
    }
)"""


# def main() -> str:
# Testar Sistemas (área 1)
#    docstring_sistemas, ids_sistemas, erro_sistemas = obter_usuarios_responsavel(1)

# Testar Infraestrutura (área 2)
#    docstring_infra, ids_infra, erro_infra = obter_usuarios_responsavel(2)

#    return f"""
#        === SISTEMAS ===
#        {docstring_sistemas}
#        IDs: {ids_sistemas}
#        Erro: {erro_sistemas}
#        === INFRAESTRUTURA ===
#        {docstring_infra}
#        IDs: {ids_infra}
#        Erro: {erro_infra}
#        """


# if __name__ == "__main__":
#    resultado = main()
# print(resultado)


""" async def main() -> str:
    return await inserir_os_infraestrutura(
        **{
            "data_solicitacao": "03/10/2025 08:31:17",
            "assunto": "Teste de gravação Infra",
            "descricao": "Este é apenas um teste de gravação Infra",
            "matSolicitante": "24142",
            "criada_por": "24142",
            "responsavel": "24142",
            "responsavel_atual": "24142",
        }
    )


if __name__ == "__main__":
    resultado = asyncio.run(main())
    print(resultado) """


""" async def main() -> str:
    return await inserir_os_sistemas(
        **{
            "assunto": "teste 1111",
            "data_solicitacao": "11/11/2025 00:00:00",
            "descricao": "testando criação de OS SIGA IA",
            "matSolicitante": "24142",
            "responsavel": "24142",
            "responsavel_atual": "24142",
            "criada_por": "24142",
            "equipe": "Equipe AVA",
            "projeto": "Operação AVA",
            "status": "Em Atendimento",
        }
    )


if __name__ == "__main__":
    resultado = asyncio.run(main())
    print(resultado) """


""" async def main() -> str:
    # Teste com todos os campos necessários
    return await inserir_os_sistemas(
        data_solicitacao="11/11/2025 10:30:00",  # Com horário específico
        assunto="teste 1111",
        descricao="testando criação de OS SIGA IA",
        responsavel="24142",
        responsavel_atual="24142",
        matSolicitante="24142",
        criada_por="24142",
        # Campos com valores padrão (você pode omitir se quiser usar os defaults)
        sistema="Sistemas AVA",
        tipo="Implementação",
        equipe="Equipe AVA",
        linguagem="PHP",
        projeto="Operação AVA",
        status="Em Atendimento",
        os_interna="Sim",
        origem="Teams",
        prioridade_usuario="Nenhuma",
        criticidade="Nenhuma",
    )


if __name__ == "__main__":
    print("=== INICIANDO TESTE ===")
    print(f"API Key configurada: {'Sim' if getenv('AVA_API_KEY') else 'Não'}")

    try:
        resultado = asyncio.run(main())
        print("=== RESULTADO ===")
        print(resultado)
    except Exception as e:
        print(f"=== ERRO NO TESTE ===")
        print(f"Erro: {e}")
        import traceback

        traceback.print_exc() """


""" async def main() -> str:
    return await listar_usuarios_responsaveis_os_siga(
        **{
            "area": "2",
        }
    )


if __name__ == "__main__":
    resultado = asyncio.run(main())
    print(resultado) """


""" async def main() -> str:
    return await listar_usuarios_equipe_por_gerente(
        **{
            "matricula_gerente": "8372",
            "descricao_equipe": "Equipe AVA",
            "situacao_usuario": "Ativo",
        }
    )


if __name__ == "__main__":
    resultado = asyncio.run(main())
    print(resultado) """


""" async def main() -> str:
    return await listar_usuarios_equipe_por_gerente(
        **{
            "matricula_gerente": "8372",
            "descricao_equipe": "Equipe AVA",
            "situacao_usuario": "Ativo",
        }
    )


if __name__ == "__main__":
    resultado = asyncio.run(main())
    print(resultado) """


""" async def main() -> str:
    return await listar_horas_trabalhadas(
        **{
            "matricula": ["24142", "14897"],
            "data_inicio": "13/10/2025",
            "data_fim": "17/10/2025",
        }
    )


if __name__ == "__main__":
    resultado = asyncio.run(main())
    print(resultado) """


""" async def main() -> str:
    return await atualizar_tempo_gasto_atendimento(
        **{
            "codigo_analista": "24142",
            "data_inicio": "22/10/2025",
        }
    )


if __name__ == "__main__":
    resultado = asyncio.run(main())
    print(resultado) """


""" async def main() -> str:
    return await inserir_atendimentos_os(
        **{
            "codigo_analista": 24142,
            "codigo_os": 182487,
            "data_inicio": "23/10/2025 08:02",
            "data_fim": "23/10/2025 08:12",
            "descricao_atendimento": "teste",
            "tipo": "Suporte Sistema",
        }
    )


if __name__ == "__main__":
    resultado = asyncio.run(main())
    print(resultado) """


""" async def main() -> str:
    return await buscar_informacoes_os(
        **{
            "codigo_os": "182487",
        }
    )


if __name__ == "__main__":
    resultado = asyncio.run(main())
    print(resultado) """


""" async def main() -> str:
    return await editar_os_sistemas(
        **{
            "codigo_os": 194987,
            "assunto": "teste",
            "criada_por": "24142",
            "data_solicitacao": "30/10/2025 15:01",
            "descricao": "testando SIGA IA",
            "matSolicitante": "24142",
            "responsavel": "24142",
            "responsavel_atual": "24142",
            "tipo": "Implementação",
            "origem": "Teams",
            "sistema": "Sistemas AVA",
            "equipe": "Equipe AVA",
            "projeto": "Operação AVA",
            "status": "Em Atendimento",
        }
    )


if __name__ == "__main__":
    resultado = asyncio.run(main())
    print(resultado) """


""" # RODAR TESTES TANTO GERAL, QUANTO PARA CANCELAMENTO E CONCLUSAO.
async def main() -> str:
    return await concluir_os_siga_ia(
        **{
            "codigo_os": "195011",
            "tipo_conclusao": "Concluída",
        }
    )


if __name__ == "__main__":
    resultado = asyncio.run(main())
    print(resultado) """


# TESTE: Buscar pendências de múltiplas matrículas concorrentemente
# EFETUA OS TESTES NA SEGUNDA PARA VER SE ESTÁ FUNCIONANDO. ELA IRÁ CONSUMIR A FUNÇÃO ORIGINAL
""" async def main() -> str:
    return await buscar_pendencias_multiplas_matriculas(
        **{
            "matriculas": [
                16500,
                24290,
                25962,
                27342,
                20634,
                23767,
                24056,
                28560,
                23724,
                29306,
                14897,
                24142,
                26302,
                29229,
                25818,
            ],  # Lista de matrículas para testar
            "dataIni": "03/11/2025",
            "dataFim": "07/11/2025",
        }
    )


if __name__ == "__main__":
    resultado = asyncio.run(main())
    print(resultado) """


# TESTE: Buscar pendências de múltiplas matrículas concorrentemente
# EFETUA OS TESTES NA SEGUNDA PARA VER SE ESTÁ FUNCIONANDO. ELA IRÁ CONSUMIR A FUNÇÃO ORIGINAL
""" async def main() -> str:
    return await buscar_pendencias_lancamentos_atendimentos(
        **{
            "matricula": 16500,
            "dataIni": "03/11/2025",
            "dataFim": "07/11/2025",
        }
    )


if __name__ == "__main__":
    resultado = asyncio.run(main())
    print(resultado) """


# testando o que está chegando no prompt, para substituir as variáveis que estão entre {}, como por exemplo as constantes.
""" def test_debug_prompt_compilation():
    #Testa a compilação do prompt com constantes

    # Adicionar o caminho correto para import
    projeto_root = os.path.dirname(os.path.dirname(__file__))  # Volta para SIGA_MCP
    siga_mcp_path = os.path.join(projeto_root, "siga_mcp")
    sys.path.append(siga_mcp_path)

    try:
        # Agora deve funcionar
        from siga_mcp.dynamic_constants import constantes_md, SYSTEM_INSTRUCTIONS

        print("Import funcionou!")

        # Salvar constantes
        with open("debug_constantes.txt", "w", encoding="utf-8") as f:
            f.write("=== CONSTANTES MARKDOWN ===\n")
            f.write(constantes_md)
            f.write("\n========================")

        # Salvar prompt compilado
        with open("debug_prompt_compilado.txt", "w", encoding="utf-8") as f:
            f.write("=== PROMPT COMPILADO ===\n")
            f.write(str(SYSTEM_INSTRUCTIONS))
            f.write("\n========================")

        print("Arquivos de debug salvos na pasta tests:")
        print("  - debug_constantes.txt")
        print("  - debug_prompt_compilado.txt")

    except ImportError as e:
        print(f"Erro de import: {e}")
        print(f"Tentando importar de: {siga_mcp_path}")

    except Exception as e:
        print(f"Outro erro: {e}")


if __name__ == "__main__":
    test_debug_prompt_compilation() """



from textwrap import dedent


def docs() -> str:
    return dedent("""\
            Busca informações detalhadas de um atendimento avulso específico.

            Esta função realiza uma consulta ao sistema SIGA através da API do AVA para obter
            todas as informações relacionadas a um atendimento avulso específico. 
            Com busca automática: Se não encontrar em avulsos, busca automaticamente em atendimentos OS.
            É especialmente útil para consultar dados antes de realizar qualquer operação no atendimento.
                  
            ORIENTAÇÃO PARA SOLICITAÇÕES GENÉRICAS:
            Esta função é específica para Atendimentos Avulsos, mas com busca automática em:
            - `buscar_informacoes_atendimentos_os` se não encontrar em avulsos
                  
            Funcionalidade de busca automática:
            Se o atendimento não for encontrado nos registros de avulsos, a função automaticamente
            tentará buscar nas tabelas de atendimentos OS, fornecendo uma experiência
            transparente ao usuário.

            Funcionalidades:
            - Consulta dados completos de um atendimento avulso pelo código e analista responsável
            - BUSCA AUTOMÁTICA em atendimentos OS se não encontrar em avulsos
            - Garante precisão na identificação do registro correto (evita ambiguidade entre códigos duplicados)
            - Retorna informações estruturadas em formato XML
            - Inclui tratamento de erros para requisições mal-sucedidas
            - Utiliza autenticação via API Key do AVA

            Args:
                codigo_atendimento (int): Código único identificador do atendimento avulso. Obrigatório. 
                    Deve ser um número inteiro válido correspondente a um atendimento existente no sistema SIGA.
                codigo_analista (str | Literal["CURRENT_USER"]): Matrícula do analista/usuário responsável pelo atendimento avulso. Obrigatório.
                    É necessário para garantir a identificação precisa do registro, evitando conflitos com códigos duplicados entre diferentes tipos de atendimento.

            Notes:
                - DIFERENCIAÇÃO DE FUNÇÃO: Esta função busca primeiro em avulsos, depois automaticamente em OS
                - BUSCA AUTOMÁTICA: Transparente ao usuário - retorna o atendimento independente do tipo
                - Requer variável de ambiente AVA_API_KEY configurada
                - A função é assíncrona e deve ser chamada com await
                - Utiliza aiohttp para requisições HTTP assíncronas
                - O XML é formatado usando a classe XMLBuilder interna
                - Ambos os parâmetros (codigo_atendimento e codigo_analista) são obrigatórios para evitar conflitos com códigos duplicados em diferentes tabelas do sistema
            """)



def docs() -> str:
    return """
        Retorna todas as constantes de referência do sistema SIGA organizadas em seções Markdown.

        Esta função fornece dados estáticos essenciais para operações no SIGA, incluindo 
        listas de projetos, sistemas, equipes, tipos de atendimento e configurações do sistema.
        Utilizada principalmente pela IA para consultas sem necessidade de chamadas API específicas.

        Funcionalidade:
        - Retorna constantes organizadas em seções com formato ## título
        - Inclui IDs e nomes no padrão **Nome:** ID  
        - Dados atualizados automaticamente conforme configuração do sistema
        - Abrange todos os tipos: sistemas, projetos, equipes, status, origens
        - Formato otimizado para parsing e consulta pela IA

        Formato de apresentação ao usuário:
        - IMPORTANTE: Remova asteriscos (**) e hashtags (##) do Markdown ao apresentar
        - Extraia apenas os dados limpos: nome e ID
        - Use tabelas HTML ou listas simples para apresentação final
        - Nunca mostre a formatação Markdown bruta para o usuário
        - Para consultas específicas, extraia APENAS a seção solicitada (ex: "## projetos")

        Exemplos de uso:
        - Usuário: "Liste os projetos" → Extrair seção "## projetos" → Apresentar como tabela limpa
        - Usuário: "Mostre os sistemas" → Extrair seção "## sistemas" → Apresentar como lista organizada  
        - Usuário: "Quais equipes existem" → Extrair seção "## equipe geral" → Apresentar como lista limpa

        Conversão de formato:
        - Raw: "**Adequações para ONA 2022:** 107"
        - Apresentar como: "Adequações para ONA 2022 (ID: 107)"
        - Ou em tabela com colunas separadas: Nome | ID

        Notes: 
            - FONTE DINÂMICA: Gerado automaticamente a partir das constantes configuradas no sistema
            - FORMATO PADRÃO: Markdown com seções ## e itens **Nome:** ID para fácil parsing
            - ATUALIZAÇÃO AUTOMÁTICA: Novas constantes adicionadas ao sistema aparecem automaticamente
            - USO PRINCIPAL: Consultas da IA para listar opções disponíveis (projetos, sistemas, equipes)
            - PERFORMANCE: Dados estáticos, sem necessidade de chamadas à API para consultas básicas
            - COMPATIBILIDADE: Funciona com todas as operações de criação/edição que precisam de constantes
            - MANUTENÇÃO: Zero manutenção - adicionar constante no sistema atualiza automaticamente
            - EXTRAÇÃO: Use a seção específica solicitada, não retorne todo o conteúdo
            - APRESENTAÇÃO: Formate como apresentação limpa sem caracteres Markdown visíveis
            
        """



def docs() -> str:
    return """
Edita as informações de um atendimento de Ordem de Serviço (OS) no sistema SIGA.

INSTRUÇÃO PARA O AGENTE IA:
ANTES de executar esta função de edição:
1. SEMPRE chame primeiro `buscar_informacoes_atendimentos_os(codigo_atendimento, codigo_analista)`
2. MOSTRE ao usuário os dados atuais vs. dados que serão alterados
3. PEÇA confirmação explícita do usuário: "Confirma as alterações? (sim/não)"
4. SÓ EXECUTE esta função se o usuário confirmar explicitamente

Esta função permite atualizar todos os campos de um atendimento existente, incluindo
datas, descrição, tipo, tempo gasto e flags de controle. Realiza validação do tipo
de atendimento e conversão automática de datas para o formato esperado pelo SIGA.

Args:
    codigo_atendimento: Código único do atendimento a ser editado
    codigo_os: Código da Ordem de Serviço à qual o atendimento pertence
    data_inicio: Data e hora de início do atendimento (formato aceito pelo converter_data_siga)
    codigo_analista: Matrícula do analista/usuário responsável pelo atendimento
    descricao_atendimento: Descrição detalhada do atendimento realizado
    tipo_atendimento: Tipo do atendimento, deve ser um dos valores válidos
    data_fim: Data e hora de fim do atendimento.
        Aceita formatos de data ou palavras-chave como "hoje", "agora", "ontem".
        Se None, será enviado como string vazia.
    primeiro_atendimento: Flag indicando se é o primeiro atendimento da OS.
    apresenta_solucao: Flag indicando se o atendimento apresenta solução.

Notes:
    - ATENÇÃO: Esta operação modifica permanentemente os dados do atendimento 
    - DIFERENCIAÇÃO DE TIPOS: Esta função é específica para atendimentos OS. Para atendimentos avulsos, use as funções específicas
    - Para busca automática: Se não encontrar nesta função, use as funções de edição de atendimento avulso com os mesmos parâmetros
    - A função realiza validação case-insensitive do tipo_atendimento
    - As datas são automaticamente convertidas usando converter_data_siga com manter_horas=True
    - A função utiliza a constante TYPE_TO_NUMBER para mapear tipos para códigos numéricos
    - Todos os parâmetros enviados são incluídos como atributos no XML de resposta
    - A API key é obtida automaticamente da variável de ambiente AVA_API_KEY
    - Em caso de falha na requisição HTTP, retorna erro interno formatado em XML
"""



import functools
import inspect
import re
import types
import importlib
from functools import wraps
from typing import (
    Any,
    Awaitable,
    Callable,
    Union,
    cast,
    get_args,
    get_origin,
    Literal,
    overload,
)

from fastmcp import FastMCP

from siga_mcp.constants import MATRICULA_USUARIO_ATUAL, MCP_TRANSPORT


def tool(
    server: FastMCP,
    transport: Literal["stdio", "http"],
) -> Callable[[Callable[..., Awaitable[Any]]], Any]:
    """
    Decorator that conditionally removes CURRENT_USER support based on MCP_TRANSPORT.

    If MCP_TRANSPORT != "stdio", removes Literal["CURRENT_USER"] from function signatures
    and cleans related documentation.
    """

    def decorator(func: Callable[..., Awaitable[Any]]) -> Any:
        # Carrega a docstring a partir de um módulo homônimo em siga_mcp.docstrings
        module_name = f"siga_mcp.docstrings.{func.__name__.lower()}"
        try:
            doc_module = importlib.import_module(module_name)
        except ModuleNotFoundError as e:
            raise ModuleNotFoundError(
                f"Arquivo de docstring não encontrado para a função '{func.__name__}'. "
                f"Esperado: siga_mcp/docstrings/{func.__name__}.py"
            ) from e

        if not hasattr(doc_module, "docs") or not callable(getattr(doc_module, "docs")):
            raise AttributeError(
                f"O módulo '{module_name}' não possui a função docs()."
            )

        try:
            raw_docstring = doc_module.docs()
            if not isinstance(raw_docstring, str):
                raise TypeError("A função docs() deve retornar uma string.")
        except Exception as e:
            raise RuntimeError(
                f"Falha ao obter docstring a partir de {module_name}.docs()."
            ) from e

        if transport == "stdio":
            # Em modo stdio, mantém a assinatura original e apenas atualiza a docstring
            # e adiciona automaticamente a resolução de CURRENT_USER via resolve_matricula
            func.__doc__ = raw_docstring
            resolved = resolve_matricula(func)
            return server.tool(resolved)

        # Remove CURRENT_USER from annotations
        def remove_current_user_from_annotation(annotation: Any) -> Any:
            if annotation == inspect.Parameter.empty or annotation is None:
                return annotation

            origin = get_origin(annotation)

            if origin is Union or (
                hasattr(types, "UnionType") and isinstance(annotation, types.UnionType)
            ):
                args = get_args(annotation)
                new_args: list[Any] = [
                    arg
                    for arg in args
                    if not (
                        get_origin(arg) is Literal and "CURRENT_USER" in get_args(arg)
                    )
                ]

                if len(new_args) == 0:
                    return str
                elif len(new_args) == 1:
                    return new_args[0]
                else:
                    if hasattr(types, "UnionType") and isinstance(
                        annotation, types.UnionType
                    ):
                        result: Any = new_args[0]
                        for arg in new_args[1:]:
                            result = result | arg
                        return result
                    else:
                        return Union[tuple(new_args)]

            if get_origin(annotation) is Literal and "CURRENT_USER" in get_args(
                annotation
            ):
                return str

            return annotation

        # Remove CURRENT_USER from docstring
        def remove_current_user_from_docstring(docstring: str | None) -> str:
            if not docstring:
                return ""

            lines: list[str] = docstring.split("\n")
            new_lines: list[str] = []
            skip_next_lines: bool = False

            for line in lines:
                if any(
                    phrase in line
                    for phrase in [
                        'If "CURRENT_USER"',
                        'Se "CURRENT_USER"',
                        'If matricula == "CURRENT_USER"',
                        '- "CURRENT_USER":',
                    ]
                ):
                    skip_next_lines = True
                    continue

                if skip_next_lines:
                    if line.strip() and (
                        line.strip().endswith(":")
                        or line.lstrip().startswith(
                            ("Args:", "Returns:", "Raises:", "Examples:", "Notes:")
                        )
                        or re.match(r"\s*\w+\s*\([^)]*\):", line)
                    ):
                        skip_next_lines = False
                    else:
                        continue

                cleaned_line: str = line
                cleaned_line = re.sub(
                    r'\s*\|\s*Literal\["CURRENT_USER"\]', "", cleaned_line
                )
                cleaned_line = re.sub(
                    r'Literal\["CURRENT_USER"\]\s*\|\s*', "", cleaned_line
                )
                cleaned_line = re.sub(r',?\s*"CURRENT_USER"[^,\n]*', "", cleaned_line)
                cleaned_line = re.sub(r'\s*-\s*"CURRENT_USER"[^-\n]*', "", cleaned_line)
                cleaned_line = re.sub(r"\s+", " ", cleaned_line)
                cleaned_line = re.sub(r"\s*\|\s*\)", ")", cleaned_line)
                cleaned_line = re.sub(r",\s*\)", ")", cleaned_line)

                new_lines.append(cleaned_line)

            result: str = "\n".join(new_lines)
            return re.sub(r"\n\s*\n\s*\n", "\n\n", result)

        # Create modified function
        sig = inspect.signature(func)
        new_params: list[inspect.Parameter] = []

        for param_name, param in sig.parameters.items():
            new_annotation: Any = remove_current_user_from_annotation(param.annotation)
            new_default: Any = (
                param.default
                if param.default != "CURRENT_USER"
                else inspect.Parameter.empty
            )
            new_param = param.replace(annotation=new_annotation, default=new_default)
            new_params.append(new_param)

        new_sig: inspect.Signature = sig.replace(parameters=new_params)
        new_docstring: str = remove_current_user_from_docstring(raw_docstring or "")

        if inspect.iscoroutinefunction(func):

            @wraps(func)
            async def _async_wrapper(*args: Any, **kwargs: Any) -> Any:
                return await func(*args, **kwargs)

            wrapped_func: Callable[..., Any] = _async_wrapper
        else:

            @wraps(func)
            def _sync_wrapper(*args: Any, **kwargs: Any) -> Any:
                return func(*args, **kwargs)

            wrapped_func = _sync_wrapper

        # Inform type checker to allow dynamic attribute set
        cast(Any, wrapped_func).__signature__ = new_sig
        wrapped_func.__doc__ = new_docstring

        new_annotations: dict[str, Any] = {}
        for param_name, param in new_sig.parameters.items():
            if param.annotation != inspect.Parameter.empty:
                new_annotations[param_name] = param.annotation

        if hasattr(func, "__annotations__") and "return" in func.__annotations__:
            new_annotations["return"] = func.__annotations__["return"]

        wrapped_func.__annotations__ = new_annotations

        # Aplica resolução automática de CURRENT_USER e garante assinatura/docstring
        resolved_func = resolve_matricula(wrapped_func)
        # Propaga assinatura/docstring modificadas para o wrapper externo
        try:
            cast(Any, resolved_func).__signature__ = new_sig
        except Exception:
            pass
        resolved_func.__doc__ = new_docstring

        return server.tool(resolved_func)

    return decorator


def resolve_matricula[T](
    func: Callable[..., Awaitable[T]],
) -> Callable[..., Awaitable[T]]:
    """
    Decorator que resolve automaticamente qualquer argumento com valor "CURRENT_USER".

    Em vez de depender do nome do parâmetro (por exemplo, 'matricula'), este decorator
    percorre todos os argumentos posicionais e nomeados e substitui qualquer ocorrência
    do valor literal "CURRENT_USER" pelo valor de MATRICULA_USUARIO_ATUAL. Isso inclui
    valores aninhados em coleções simples (list, tuple, set, dict - apenas valores).

    Regras:
    1) A substituição só ocorre quando MCP_TRANSPORT == "stdio"; caso contrário, lança erro
       se for detectado ao menos um "CURRENT_USER" nos argumentos.
    2) Somente valores exatamente iguais à string "CURRENT_USER" são substituídos (não faz
       match parcial dentro de outras strings).

    Funciona com funções assíncronas.
    """

    def _replace_current_user(value: Any) -> Any:
        # Substitui apenas valores exatamente iguais a "CURRENT_USER"
        if isinstance(value, str):
            return MATRICULA_USUARIO_ATUAL if value == "CURRENT_USER" else value
        # Coleções comuns: processa recursivamente
        if isinstance(value, list):
            result_list: list[Any] = []
            for v in cast(list[Any], value):
                result_list.append(_replace_current_user(v))
            return result_list
        if isinstance(value, tuple):
            result_tuple_list: list[Any] = []
            for v in cast(tuple[Any, ...], value):
                result_tuple_list.append(_replace_current_user(v))
            return tuple(result_tuple_list)
        if isinstance(value, set):
            result_set: set[Any] = set()
            for v in cast(set[Any], value):
                result_set.add(_replace_current_user(v))
            return result_set
        if isinstance(value, dict):
            # Apenas valores; chaves permanecem inalteradas
            result_dict: dict[Any, Any] = {}
            for k, v in cast(dict[Any, Any], value).items():
                result_dict[k] = _replace_current_user(v)
            return result_dict
        return value

    def _has_current_user(value: Any) -> bool:
        if isinstance(value, str):
            return value == "CURRENT_USER"
        if isinstance(value, list):
            for v in cast(list[Any], value):
                if _has_current_user(v):
                    return True
            return False
        if isinstance(value, tuple):
            for v in cast(tuple[Any, ...], value):
                if _has_current_user(v):
                    return True
            return False
        if isinstance(value, set):
            for v in cast(set[Any], value):
                if _has_current_user(v):
                    return True
            return False
        if isinstance(value, dict):
            for v in cast(dict[Any, Any], value).values():
                if _has_current_user(v):
                    return True
            return False
        return False

    @functools.wraps(func)
    async def wrapper(*args: Any, **kwargs: Any) -> T:
        # Obtém a assinatura da função para mapear argumentos posicionais
        sig = inspect.signature(func)
        bound_args = sig.bind(*args, **kwargs)
        bound_args.apply_defaults()

        # Verifica se existe qualquer ocorrência de "CURRENT_USER" em qualquer argumento
        any_current_user = any(
            _has_current_user(val) for val in bound_args.arguments.values()
        )

        if any_current_user:
            from siga_mcp.constants import MCP_TRANSPORT

            if MCP_TRANSPORT != "stdio":
                raise ValueError(
                    "MCP_TRANSPORT must be stdio to get current AD user (found 'CURRENT_USER' in arguments)"
                )

            # Substitui "CURRENT_USER" por MATRICULA_USUARIO_ATUAL em todos os argumentos
            for name, val in list(bound_args.arguments.items()):
                new_val = _replace_current_user(val)
                bound_args.arguments[name] = new_val

        # Chama a função original com os argumentos atualizados
        return await func(*bound_args.args, **bound_args.kwargs)

    return wrapper


@overload
def controlar_acesso_matricula[T](
    func: Callable[..., Awaitable[T]],
) -> Callable[..., Awaitable[Union[T, str]]]: ...


@overload
def controlar_acesso_matricula[T](
    func: Callable[..., T],
) -> Callable[..., Union[T, str]]: ...


def controlar_acesso_matricula[T](
    func: Callable[..., Union[T, Awaitable[T]]],
) -> Callable[..., Union[T, str, Awaitable[Union[T, str]]]]:
    """
    Decorator que verifica automaticamente permissões de acesso à matrícula.

    Este decorator intercepta chamadas de função e verifica se o usuário atual
    tem permissão para acessar os dados da matrícula especificada:
    1. Obtém o parâmetro 'matricula' da função
    2. Verifica permissão usando verificar_permissao_acesso_matricula()
    3. Retorna erro 401 se não houver permissão
    4. Permite execução da função se houver permissão

    Funciona com funções síncronas e assíncronas que tenham o parâmetro 'matricula'.
    """
    if MCP_TRANSPORT == "http":
        # Em modo HTTP, não aplica controle de acesso
        return func

    # Detecta se a função é assíncrona
    is_async = inspect.iscoroutinefunction(func)

    if is_async:

        @functools.wraps(func)
        async def async_wrapper(*args: Any, **kwargs: Any) -> Union[T, str]:
            # Obtém a assinatura da função para mapear argumentos posicionais
            sig = inspect.signature(func)
            bound_args = sig.bind(*args, **kwargs)
            bound_args.apply_defaults()

            # Verifica se existe o parâmetro 'matricula'
            if "matricula" in bound_args.arguments:
                from siga_mcp.utils import verificar_permissao_acesso_matricula

                matricula = bound_args.arguments["matricula"]

                if matricula == "CURRENT_USER":
                    matricula = MATRICULA_USUARIO_ATUAL

                # Verifica se o usuário tem permissão para acessar esta matrícula
                usuario_nao_pode_acessar = not verificar_permissao_acesso_matricula(
                    matricula
                )

                if usuario_nao_pode_acessar:
                    # Retorna o nome da função para contextualizar o erro
                    func_name = func.__name__
                    return (
                        f"Erro 401: Permissão negada para '{func_name}'. Usuário atual logado é diferente do solicitado. "
                        "Você não pode acessar essa função. Contate seu administrador."
                    )

            # Chama a função original se tiver permissão
            return await func(*args, **kwargs)  # type: ignore

        return async_wrapper  # type: ignore
    else:

        @functools.wraps(func)
        def sync_wrapper(*args: Any, **kwargs: Any) -> Union[T, str]:
            # Obtém a assinatura da função para mapear argumentos posicionais
            sig = inspect.signature(func)
            bound_args = sig.bind(*args, **kwargs)
            bound_args.apply_defaults()

            # Verifica se existe o parâmetro 'matricula'
            if "matricula" in bound_args.arguments:
                from siga_mcp.utils import verificar_permissao_acesso_matricula

                matricula = bound_args.arguments["matricula"]

                if matricula == "CURRENT_USER":
                    matricula = MATRICULA_USUARIO_ATUAL

                # Verifica se o usuário tem permissão para acessar esta matrícula
                usuario_nao_pode_acessar = not verificar_permissao_acesso_matricula(
                    matricula
                )

                if usuario_nao_pode_acessar:
                    # Retorna o nome da função para contextualizar o erro
                    func_name = func.__name__
                    return (
                        f"Erro 401: Permissão negada para '{func_name}'. Usuário atual logado é diferente do solicitado. "
                        "Você não pode acessar essa função. Contate seu administrador."
                    )

            # Chama a função original se tiver permissão
            return func(*args, **kwargs)  # type: ignore

        return sync_wrapper  # type: ignore
